/* Copyright 2008 Mike Mammarella. Released under GPLv2. */

#include <stdlib.h>
#include <gccore.h>
#include <inttypes.h>
#include <wiiuse/wpad.h>

#include "GRRLIB/GRRLIB.h"
#include "wpad_leds.h"

#include "font16_png.h"

Mtx GXmodelView2D;

int main(void)
{
	uint8_t * default_font;
	int led = 0, led_dir = 1, led_timer = 10;
	int i, motes[4] = {80, 80, 80, 80};
	
	VIDEO_Init();
	WPAD_Init();
	
	GRRLIB_InitVideo();
	GRRLIB_Start();
	
	default_font = GRRLIB_LoadTexture(font16_png);
	
	while(WPAD_Probe(0, NULL) != WPAD_ERR_NONE)
	{
		WPAD_ScanPads();
		
		GRRLIB_FillScreen(0xFF000000);
		GRRLIB_Printf(20, 440, default_font, 0xFFFFFFFF, 1, "Built " __DATE__ " " __TIME__);
		GRRLIB_Render();
	}
	
	for(;;)
	{
		int count = 0;
		
		WPAD_ScanPads();
		
		for(i = 0; i < 4; i++)
		{
			int mote = (WPAD_Probe(i, NULL) == WPAD_ERR_NONE);
			if(mote)
			{
				if(motes[i])
					motes[i]--;
			}
			else
				motes[i] = 80;
			if(!motes[i])
			{
				if(WPAD_ButtonsUp(i) & WPAD_BUTTON_B)
				{
					WPAD_Disconnect(i);
					motes[i] = 80;
				}
				else
					count++;
			}
		}
		if(count)
		{
			if(led >= count * 4)
			{
				led = count * 4 - 2;
				led_dir = 1;
				led_timer = 1;
			}
			if(!--led_timer)
			{
				int idx;
				led += led_dir;
				if(!led || led == count * 4 - 1)
					led_dir = -led_dir;
				idx = led;
				for(i = 0; i < 4; i++)
				{
					if(motes[i])
						continue;
					if(0 <= idx && idx < 4)
						WPAD_SetLeds(i, WIIMOTE_LED_1 << idx);
					else
						WPAD_SetLeds(i, 0);
					idx -= 4;
				}
				led_timer = 12 - count * 2;
			}
		}
		else
		{
			led = 0;
			led_dir = 1;
			led_timer = 10;
		}
		
		GRRLIB_FillScreen(0xFF000000);
		GRRLIB_Printf(104, 220, default_font, 0xFFFFFFFF, 1, "Press HOME to exit, or (B)");
		GRRLIB_Printf(104, 240, default_font, 0xFFFFFFFF, 1, "to disconnect a wiimote.");
		GRRLIB_Render();
		
		for(i = 0; i < 4; i++)
			if(WPAD_ButtonsUp(i) & WPAD_BUTTON_HOME)
				break;
		if(i < 4)
			break;
	}
	
	for(i = 0; i < 4; i++)
		WPAD_Disconnect(i);
	
	for(i = 0; i < 3; i++)
	{
		GRRLIB_FillScreen(0xFF000000);
		GRRLIB_Printf(142, 220, default_font, 0xFFFFFFFF, 1, "Returning to loader...");
		GRRLIB_Render();
	}
	
	free(default_font);
	
	return 0;
}
